@@ -15,7 +15,7 @@ char *egl_error(char *where) {
 /* Sets up an OpenGL ES 2 context. Returnes NULL if it succeeds, or
  * an error message on failure.
  */
-char *egl_init(int interval) {
+char *egl_init(SDL_Window *sdl_window, int interval) {
 	return NULL;
 }
 
