@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -181,6 +181,7 @@ for p in generate_styles.prefixes:
     cython("renpy.styledata.style_{}functions".format(p), pyx=setuplib.gen + "/style_{}functions.pyx".format(p))
 
 # renpy.display
+cython("renpy.display.matrix")
 cython("renpy.display.render", libs=[ 'z', 'm' ])
 cython("renpy.display.accelerator", libs=sdl + [ 'z', 'm' ])
 
@@ -216,12 +217,18 @@ cython("renpy.gl.glenviron_limited", libs=glew_libs, compile_if=not gl2_only)
 cython("renpy.gl.glrtt_copy", libs=glew_libs)
 cython("renpy.gl.glrtt_fbo", libs=glew_libs)
 
+cython("renpy.gl2.uguugl", libs=sdl)
+cython("renpy.gl2.uguu", libs=sdl)
+cython("renpy.gl2.gl2geometry")
+cython("renpy.gl2.gl2draw", libs=sdl)
+cython("renpy.gl2.gl2texture", libs=sdl)
+cython("renpy.gl2.gl2shader", libs=sdl)
+
 if not (android or ios or emscripten):
     # renpy.angle
     def anglecopy(fn):
         copyfile("renpy/gl/" + fn, "renpy/angle/" + fn, "DEF ANGLE = False", "DEF ANGLE = True")
 
-    anglecopy("glblacklist.py")
     anglecopy("gldraw.pxd")
     anglecopy("gldraw.pyx")
     anglecopy("glenviron_shader.pyx")
