@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -45,11 +45,19 @@ emscripten = "RENPY_EMSCRIPTEN" in os.environ
 # Is coverage enabled?
 coverage = "RENPY_COVERAGE" in os.environ
 
+# Are we doing a static build?
+static = "RENPY_STATIC" in os.environ
+
 if coverage:
     gen = "gen.coverage"
 else:
     gen = "gen"
 
+
+if static:
+    gen += "-static"
+
+
 # The cython command.
 cython_command = os.environ.get("RENPY_CYTHON", "cython")
 
@@ -317,7 +325,19 @@ def cython(name, source=[], libs=[], includes=[], compile_if=True, define_macros
                 "-o",
                 c_fn])
 
-        except subprocess.CalledProcessError, e:
+            # Fix-up source for static loading
+            if static and (len(split_name) > 1):
+                parent_module = '.'.join(split_name[:-1])
+                parent_module_identifier = parent_module.replace('.', '_')
+                with open(c_fn, 'r') as f:
+                    ccode = f.read()
+                ccode = re.sub('Py_InitModule4\("([^"]+)"', 'Py_InitModule4("'+parent_module+'.\\1"', ccode)
+                ccode = re.sub('^__Pyx_PyMODINIT_FUNC init', '__Pyx_PyMODINIT_FUNC init'+parent_module_identifier+'_', ccode, 0, re.MULTILINE)  # Cython 0.28.2
+                ccode = re.sub('^PyMODINIT_FUNC init', 'PyMODINIT_FUNC init'+parent_module_identifier+'_', ccode, 0, re.MULTILINE)  # Cython 0.25.2
+                with open(c_fn, 'w') as f:
+                    f.write(ccode)
+
+        except subprocess.CalledProcessError as e:
             print()
             print(str(e))
             print()
