@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,6 +21,7 @@
 
 # This contains various Displayables that handle events.
 
+from __future__ import print_function
 
 import renpy.display
 import renpy.audio
@@ -76,43 +77,58 @@ def compile_event(key, keydown):
     if part[0] == "joy" or part[0] == "pad":
         return "(False)"
 
+    MODIFIERS = { "keydown", "keyup", "repeat", "alt", "meta", "shift", "noshift", "ctrl" }
+    modifiers = set()
+
+    while part[0] in MODIFIERS:
+        modifiers.add(part.pop(0))
+
+    key = "_".join(part)
+
+    if "keydown" in modifiers:
+        keydown = True
+    elif "keyup" in modifiers:
+        keydown = False
+
     # Otherwise, deal with it as a key.
     if keydown:
         rv = "(ev.type == %d" % pygame.KEYDOWN
     else:
         rv = "(ev.type == %d" % pygame.KEYUP
 
-    MODIFIERS = { "repeat", "alt", "meta", "shift", "noshift", "ctrl" }
-    modifiers = set()
-
-    while part[0] in MODIFIERS:
-        modifiers.add(part.pop(0))
-
     if "repeat" in modifiers:
         rv += " and (ev.repeat)"
     else:
         rv += " and (not ev.repeat)"
 
-    if "alt" in modifiers:
-        rv += " and (ev.mod & %d)" % pygame.KMOD_ALT
-    else:
-        rv += " and not (ev.mod & %d)" % pygame.KMOD_ALT
+    if key not in [ "K_LALT", "K_RALT" ]:
 
-    if "meta" in modifiers:
-        rv += " and (ev.mod & %d)" % pygame.KMOD_META
-    else:
-        rv += " and not (ev.mod & %d)" % pygame.KMOD_META
+        if "alt" in modifiers:
+            rv += " and (ev.mod & %d)" % pygame.KMOD_ALT
+        else:
+            rv += " and not (ev.mod & %d)" % pygame.KMOD_ALT
 
-    if "ctrl" in modifiers:
-        rv += " and (ev.mod & %d)" % pygame.KMOD_CTRL
-    else:
-        rv += " and not (ev.mod & %d)" % pygame.KMOD_CTRL
+    if key not in [ "K_LGUI", "K_RGUI" ]:
 
-    if "shift" in modifiers:
-        rv += " and (ev.mod & %d)" % pygame.KMOD_SHIFT
+        if "meta" in modifiers:
+            rv += " and (ev.mod & %d)" % pygame.KMOD_META
+        else:
+            rv += " and not (ev.mod & %d)" % pygame.KMOD_META
 
-    if "noshift" in modifiers:
-        rv += " and not (ev.mod & %d)" % pygame.KMOD_SHIFT
+    if key not in [ "K_LCTRL", "K_RCTRL" ]:
+
+        if "ctrl" in modifiers:
+            rv += " and (ev.mod & %d)" % pygame.KMOD_CTRL
+        else:
+            rv += " and not (ev.mod & %d)" % pygame.KMOD_CTRL
+
+    if key not in [ "K_LSHIFT", "K_RSHIFT" ]:
+
+        if "shift" in modifiers:
+            rv += " and (ev.mod & %d)" % pygame.KMOD_SHIFT
+
+        if "noshift" in modifiers:
+            rv += " and not (ev.mod & %d)" % pygame.KMOD_SHIFT
 
     if len(part) == 1:
         if len(part[0]) != 1:
@@ -130,8 +146,6 @@ def compile_event(key, keydown):
             else:
                 return "(False)"
 
-        key = "_".join(part)
-
         rv += " and ev.key == %d)" % (getattr(pygame.constants, key))
 
     return rv
@@ -342,8 +356,26 @@ def run_periodic(var, st):
         return var.periodic(st)
 
 
+def get_tooltip(action):
+
+    if isinstance(action, (list, tuple)):
+        for i in action:
+            rv = get_tooltip(i)
+            if rv is not None:
+                return rv
+
+        return None
+
+    func = getattr(action, "get_tooltip", None)
+    if func is None:
+        return None
+
+    return func()
+
+
 def is_selected(action):
     """
+    :name: renpy.is_selected
     :doc: run
 
     Returns true if `action` indicates it is selected, or false otherwise.
@@ -363,6 +395,7 @@ def is_selected(action):
 
 def is_sensitive(action):
     """
+    :name: renpy.is_sensitive
     :doc: run
 
     Returns true if `action` indicates it is sensitive, or False otherwise.
@@ -543,7 +576,13 @@ class SayBehavior(renpy.display.layout.Null):
 
     def set_text(self, text):
         self.text = text
-        self.afm_length = max(text.end - text.start, 1)
+
+        try:
+            afm_text = text.text[0][text.start:text.end]
+            afm_text = renpy.text.extras.filter_text_tags(afm_text, allow=[])
+            self.afm_length = max(len(afm_text), 1)
+        except:
+            self.afm_length = max(text.end - text.start, 1)
 
     def event(self, ev, x, y, st):
 
@@ -581,7 +620,7 @@ class SayBehavior(renpy.display.layout.Null):
 
                 if not renpy.config.enable_rollback_side:
                     rollback_side = "disable"
-                if renpy.mobile:
+                if renpy.exports.mobile:
                     rollback_side = renpy.game.preferences.mobile_rollback_side
                 else:
                     rollback_side = renpy.game.preferences.desktop_rollback_side
@@ -628,7 +667,7 @@ class SayBehavior(renpy.display.layout.Null):
                 elif renpy.game.context().seen_current(True):
                     return True
                 else:
-                    renpy.config.skipping = False
+                    renpy.config.skipping = None
                     renpy.exports.restart_interaction()
 
             else:
@@ -663,6 +702,9 @@ class Button(renpy.display.layout.Window):
     keysym = None
     alternate_keysym = None
 
+    # This locks the displayable against further change.
+    locked = False
+
     def __init__(self, child=None, style='button', clicked=None,
                  hovered=None, unhovered=None, action=None, role=None,
                  time_policy=None, keymap={}, alternate=None,
@@ -695,8 +737,25 @@ class Button(renpy.display.layout.Window):
         self._duplicatable = False
 
     def _duplicate(self, args):
+        if args and args.args:
+            args.extraneous()
+
         return self
 
+    def _get_tooltip(self):
+        if self._tooltip is not None:
+            return self._tooltip
+
+        return get_tooltip(self.action)
+
+    def _in_current_store(self):
+        rv = self._copy()
+        rv.style = self.style.copy()
+        rv.set_style_prefix(self.style.prefix, True)
+        rv.focusable = False
+        rv.locked = True
+        return rv
+
     def predict_one_action(self):
         predict_action(self.clicked)
         predict_action(self.hovered)
@@ -792,40 +851,46 @@ class Button(renpy.display.layout.Window):
 
     def per_interact(self):
 
-        if self.action is not None:
-            if self.is_selected():
-                role = 'selected_'
+        if not self.locked:
+
+            if self.action is not None:
+                if self.is_selected():
+                    role = 'selected_'
+                else:
+                    role = ''
+
+                if self.is_sensitive():
+                    clicked = self.action
+                else:
+                    clicked = None
+                    role = ''
+
             else:
                 role = ''
+                clicked = self.clicked
 
-            if self.is_sensitive():
-                clicked = self.action
+            if self.role_parameter is not None:
+                role = self.role_parameter
+
+            if (role != self.role) or (clicked is not self.clicked):
+                renpy.display.render.invalidate(self)
+                self.role = role
+                self.clicked = clicked
+
+            if self.clicked is not None:
+                self.set_style_prefix(self.role + "idle_", True)
+                self.focusable = True
             else:
-                clicked = None
-
-        else:
-            role = ''
-            clicked = self.clicked
-
-        if self.role_parameter is not None:
-            role = self.role_parameter
-
-        if (role != self.role) or (clicked is not self.clicked):
-            renpy.display.render.invalidate(self)
-            self.role = role
-            self.clicked = clicked
-
-        if self.clicked is not None:
-            self.set_style_prefix(self.role + "idle_", True)
-            self.focusable = True
-        else:
-            self.set_style_prefix(self.role + "insensitive_", True)
-            self.focusable = False
+                self.set_style_prefix(self.role + "insensitive_", True)
+                self.focusable = False
 
         super(Button, self).per_interact()
 
     def event(self, ev, x, y, st):
 
+        if self.locked:
+            return None
+
         def handle_click(action):
             renpy.exports.play(self.style.activate_sound)
 
@@ -889,7 +954,7 @@ class Button(renpy.display.layout.Window):
         if (self.clicked is not None) and map_event(ev, "button_ignore"):
             raise renpy.display.core.IgnoreEvent()
 
-        if (self.clicked is not None) and map_event(ev, "button_alternate_ignore"):
+        if (self.alternate is not None) and map_event(ev, "button_alternate_ignore"):
             raise renpy.display.core.IgnoreEvent()
 
         # If clicked,
@@ -909,7 +974,12 @@ class Button(renpy.display.layout.Window):
         return ""
 
     def _tts_all(self):
-        return self._tts_common(alt(self.action))
+        rv = self._tts_common(alt(self.action))
+
+        if self.is_selected():
+            rv += " " + renpy.minstore.__("selected")
+
+        return rv
 
 # Reimplementation of the TextButton widget as a Button and a Text
 # widget.
@@ -918,8 +988,10 @@ class Button(renpy.display.layout.Window):
 def TextButton(text, style='button', text_style='button_text',
                clicked=None, **properties):
 
-    text = renpy.text.text.Text(text, style=text_style)  # @UndefinedVariable
-    return Button(text, style=style, clicked=clicked, **properties)
+    text_properties, button_properties = renpy.easy.split_properties(properties, "text_", "")
+
+    text = renpy.text.text.Text(text, style=text_style, **text_properties)  # @UndefinedVariable
+    return Button(text, style=style, clicked=clicked, **button_properties)
 
 
 class ImageButton(Button):
@@ -1055,6 +1127,7 @@ class Input(renpy.text.text.Text):  # @UndefinedVariable
     default = u""
     edit_text = u""
     value = None
+    shown = False
 
     def __init__(self,
                  default="",
@@ -1070,6 +1143,7 @@ class Input(renpy.text.text.Text):  # @UndefinedVariable
                  editable=True,
                  pixel_width=None,
                  value=None,
+                 copypaste=False,
                  **properties):
 
         super(Input, self).__init__("", style=style, replaces=replaces, substitute=False, **properties)
@@ -1088,6 +1162,7 @@ class Input(renpy.text.text.Text):  # @UndefinedVariable
         self.exclude = exclude
         self.prefix = prefix
         self.suffix = suffix
+        self.copypaste = copypaste
 
         self.changed = changed
 
@@ -1113,15 +1188,10 @@ class Input(renpy.text.text.Text):  # @UndefinedVariable
             self.content = replaces.content
             self.editable = replaces.editable
             self.caret_pos = replaces.caret_pos
+            self.shown = replaces.shown
 
         self.update_text(self.content, self.editable)
 
-    def _show(self):
-        if self.default != self.content:
-            self.content = self.default
-            self.caret_pos = len(self.content)
-            self.update_text(self.content, self.editable)
-
     def update_text(self, new_content, editable, check_size=False):
 
         edit = renpy.display.interface.text_editing
@@ -1214,6 +1284,18 @@ class Input(renpy.text.text.Text):  # @UndefinedVariable
             if self.value.default and (default_input_value is None):
                 default_input_value = self.value
 
+        if not self.shown:
+
+            if self.value is not None:
+                default = self.value.get_text()
+                self.default = unicode(default)
+
+            self.content = self.default
+            self.caret_pos = len(self.content)
+            self.update_text(self.content, self.editable)
+
+            self.shown = True
+
     def event(self, ev, x, y, st):
 
         self.old_caret_pos = self.caret_pos
@@ -1221,7 +1303,7 @@ class Input(renpy.text.text.Text):  # @UndefinedVariable
         if not self.editable:
             return None
 
-        if pygame.key.get_mods() & pygame.KMOD_LALT:
+        if (ev.type == pygame.KEYDOWN) and (pygame.key.get_mods() & pygame.KMOD_LALT) and (not ev.unicode):
             return None
 
         l = len(self.content)
@@ -1287,6 +1369,17 @@ class Input(renpy.text.text.Text):  # @UndefinedVariable
             renpy.display.render.redraw(self, 0)
             raise renpy.display.core.IgnoreEvent()
 
+        elif self.copypaste and map_event(ev, "input_copy"):
+            pygame.scrap.put(pygame.scrap.SCRAP_TEXT, self.content)
+            raise renpy.display.core.IgnoreEvent()
+
+        elif self.copypaste and map_event(ev, "input_paste"):
+            text = pygame.scrap.get(pygame.scrap.SCRAP_TEXT)
+            raw_text = ""
+            for c in text:
+                if ord(c) >= 32:
+                    raw_text += c
+
         elif ev.type == pygame.TEXTEDITING:
             self.update_text(self.content, self.editable, check_size=True)
 
@@ -1301,7 +1394,10 @@ class Input(renpy.text.text.Text):  # @UndefinedVariable
             if ev.unicode and ord(ev.unicode[0]) >= 32:
                 raw_text = ev.unicode
             elif renpy.display.interface.text_event_in_queue():
-                raw_text = ''
+                raise renpy.display.core.IgnoreEvent()
+            elif (32 <= ev.key < 127) and not (ev.mod & (pygame.KMOD_ALT | pygame.KMOD_META)):
+                # Ignore printable keycodes without unicode.
+                raise renpy.display.core.IgnoreEvent()
 
         if raw_text is not None:
 
@@ -1359,7 +1455,9 @@ class Adjustment(renpy.object.Object):
 
     """
 
-    def __init__(self, range=1, value=0, step=None, page=None, changed=None, adjustable=None, ranged=None):  # @ReservedAssignment
+    force_step = False
+
+    def __init__(self, range=1, value=0, step=None, page=None, changed=None, adjustable=None, ranged=None, force_step=False):  # @ReservedAssignment
         """
         The following parameters correspond to fields or properties on
         the adjustment object:
@@ -1387,12 +1485,12 @@ class Adjustment(renpy.object.Object):
         The following parameters control the behavior of the adjustment.
 
         `adjustable`
-             If True, this adjustment can be changed by a bar. If False,
-             it can't.
+            If True, this adjustment can be changed by a bar. If False,
+            it can't.
 
-             It defaults to being adjustable if a `changed` function
-             is given or if the adjustment is associated with a viewport,
-             and not adjustable otherwise.
+            It defaults to being adjustable if a `changed` function
+            is given or if the adjustment is associated with a viewport,
+            and not adjustable otherwise.
 
         `changed`
             This function is called with the new value when the value of
@@ -1402,6 +1500,16 @@ class Adjustment(renpy.object.Object):
             This function is called with the adjustment object when
             the range of the adjustment is set by a viewport.
 
+        `force_step`
+            If True and this adjustment changes by dragging associated
+            viewport or a bar, value will be changed only if the drag
+            reached next step.
+            If "release" and this adjustment changes by dragging associated
+            viewport or a bar, after the release, value will be
+            rounded to the nearest step.
+            If False, this adjustment will changes by dragging, ignoring
+            the step value.
+
         .. method:: change(value)
 
             Changes the value of the adjustment to `value`, updating
@@ -1421,9 +1529,27 @@ class Adjustment(renpy.object.Object):
         self.changed = changed
         self.adjustable = adjustable
         self.ranged = ranged
+        self.force_step = force_step
+
+    def round_value(self, value, release):
+        # Prevent deadlock border points
+        if value <= 0:
+            return 0
+        elif value >= self._range:
+            return self._range
+
+        if self.force_step is False:
+            return value
+
+        if (not release) and self.force_step == "release":
+            return value
+
+        return type(self.value)(self.step * round(float(value) / self.step))
 
     def get_value(self):
-        if self._value > self._range:
+        if self._value <= 0:
+            return 0
+        if self._value >= self._range:
             return self._range
 
         return self._value
@@ -1498,7 +1624,7 @@ class Adjustment(renpy.object.Object):
         """
 
         for d in adj_registered.setdefault(self, [ ]):
-            renpy.display.render.redraw(d, 0)
+            renpy.display.render.invalidate(d)
 
 
 class Bar(renpy.display.core.Displayable):
@@ -1550,6 +1676,10 @@ class Bar(renpy.display.core.Displayable):
                 adjustment = value.get_adjustment()
                 renpy.game.interface.timeout(0)
 
+                tooltip = value.get_tooltip()
+                if tooltip is not None:
+                    properties.setdefault("tooltip", tooltip)
+
             else:
                 adjustment = Adjustment(range, value, step=step, page=page, changed=changed)
 
@@ -1597,9 +1727,10 @@ class Bar(renpy.display.core.Displayable):
         self.focusable = self.adjustment.adjustable
         self.adjustment.register(self)
 
-    def predict_one(self):
-        pd = renpy.display.predict.displayable
-        self.style._predict_window(pd)
+    def visit(self):
+        rv = [ ]
+        self.style._visit_bar(rv.append)
+        return rv
 
     def render(self, width, height, st, at):
 
@@ -1766,7 +1897,7 @@ class Bar(renpy.display.core.Displayable):
         ignore_event = False
 
         if not grabbed and map_event(ev, "bar_activate"):
-            renpy.display.tts.speak("activate")
+            renpy.display.tts.speak(renpy.minstore.__("activate"))
             renpy.display.focus.set_grab(self)
             self.set_style_prefix("selected_hover_", True)
             just_grabbed = True
@@ -1783,12 +1914,12 @@ class Bar(renpy.display.core.Displayable):
                 decrease = "bar_left"
 
             if map_event(ev, decrease):
-                renpy.display.tts.speak("decrease")
+                renpy.display.tts.speak(renpy.minstore.__("decrease"))
                 value -= self.adjustment.step
                 ignore_event = True
 
             if map_event(ev, increase):
-                renpy.display.tts.speak("increase")
+                renpy.display.tts.speak(renpy.minstore.__("increase"))
                 value += self.adjustment.step
                 ignore_event = True
 
@@ -1819,21 +1950,32 @@ class Bar(renpy.display.core.Displayable):
                 value = int(value)
 
             if value < 0:
+                renpy.display.tts.speak("")
                 value = 0
 
             if value > range:
+                renpy.display.tts.speak("")
                 value = range
 
         if invert:
             value = range - value
 
         if grabbed and not just_grabbed and map_event(ev, "bar_deactivate"):
-            renpy.display.tts.speak("deactivate")
+            renpy.display.tts.speak(renpy.minstore.__("deactivate"))
             self.set_style_prefix("hover_", True)
             renpy.display.focus.set_grab(None)
-            ignore_event = True
+
+            # Invoke rounding adjustment on bar release
+            value = self.adjustment.round_value(value, release=True)
+            if value != old_value:
+                rv = self.adjustment.change(value)
+                if rv is not None:
+                    return rv
+
+            raise renpy.display.core.IgnoreEvent()
 
         if value != old_value:
+            value = self.adjustment.round_value(value, release=False)
             rv = self.adjustment.change(value)
             if rv is not None:
                 return rv
@@ -1855,9 +1997,9 @@ class Bar(renpy.display.core.Displayable):
         if self.value is not None:
             alt = self.value.alt
         else:
-            alt = "Bar"
+            alt = ""
 
-        return self._tts_common(alt)
+        return self._tts_common(alt) + renpy.minstore.__("bar")
 
 
 class Conditional(renpy.display.layout.Container):
@@ -2052,14 +2194,20 @@ class OnEvent(renpy.display.core.Displayable):
         self.event_name = event
         self.action = action
 
+    def is_event(self, event):
+        if isinstance(self.event_name, basestring):
+            return self.event_name == event
+        else:
+            return event in self.event_name
+
     def _handles_event(self, event):
-        if self.event_name == event:
+        if self.is_event(event):
             return True
         else:
             return False
 
     def set_transform_event(self, event):
-        if event == self.event_name:
+        if self.is_event(event):
             run(self.action)
 
     def render(self, width, height, st, at):
