@@ -199,6 +199,13 @@ def write_keywords():
 line_buffer = collections.defaultdict(list)
 
 
+# A map from id(o) to the names it's documented under.
+documented = collections.defaultdict(list)
+
+# This keeps all objectsd we see alive, to prevent duplicates in documented.
+documented_list = [ ]
+
+
 def scan(name, o, prefix=""):
 
     doc_type = "function"
@@ -297,6 +304,9 @@ def scan(name, o, prefix=""):
         for i in dir(o):
             scan(i, getattr(o, i), prefix + "    ")
 
+    documented_list.append(o)
+    documented[id(o)].append(name)
+
 
 def scan_section(name, o):
     """
@@ -447,3 +457,17 @@ def write_tq():
 
     with open("source/thequestion.rst", "w") as f:
         f.write(template.format(script=script, options=options))
+
+
+def check_dups():
+
+    duplicates = False
+
+    for v in documented.values():
+        if len(v) >= 2:
+            duplicates = True
+
+            print(" and ".join(v), "are duplicate.")
+
+    if duplicates:
+        raise SystemExit(1)
